------------------------------------------------------------------------------
-- MakeMeGetMyMain - Alarms you when something happens to your character
------------------------------------------------------------------------------
-- MakeMeGetMyMain.lua
--
-- Author: Expelliarm5s / January 2023 / All Rights Reserved
--
-- Version 1.1.24
------------------------------------------------------------------------------
-- luacheck: ignore 212 globals DLAPI

local addonName, addon = ...
addon = LibStub("AceAddon-3.0"):NewAddon(addon, addonName, "AceEvent-3.0", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local AceConfigRegistry = LibStub("AceConfigRegistry-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceDB = LibStub("AceDB-3.0")
local LDB = LibStub("LibDataBroker-1.1")
local LibQTip = LibStub("LibQTip-1.0")
-- local private = {}
------------------------------------------------------------------------------

------------------------------------------------------------------------------
-- General Settings

addon.METADATA = {
	NAME = GetAddOnMetadata(..., "Title"),
	VERSION = GetAddOnMetadata(..., "Version"),
	NOTES = GetAddOnMetadata(..., "Notes"),
}

------------------------------------------------------------------------------
-- Debug Stuff

function addon:DebugLog(...)
	-- external
	if DLAPI then DLAPI.DebugLog(addonName, ...) end
end

function addon:DebugPrintf(...)
	if addon.isDebug then
		local status, res = pcall(format, ...)
		if status then
			addon:DebugLog(res)
		end
	end
end

function addon:ToggleDebug()
	addon.isDebug = not addon.isDebug
	if (not addon.isDebug) then
		addon:Printf(L["Debug is off"])
		addon:DebugPrintf(L["Debug is off"])
	else
		addon:Printf(L["Debug is on"])
		addon:DebugPrintf(L["Debug is on"])
	end
end

------------------------------------------------------------------------------
-- Addon Initialization

function addon:EnableCmd()
	addon:Printf(L["ON"])
	addon.isEnabled = true
	addon.db.global.enabled = addon.isEnabled
	addon:Update()
	addon.GUI:Close()
end

function addon:DisableCmd()
	addon:Printf(L["OFF"])
	addon.isEnabled = false
	addon.db.global.enabled = addon.isEnabled
	addon:Update()
	addon.GUI:Close()
end

function addon:ChatCommand(str)
	addon:DebugPrintf("ChatCommand(%s)", tostring(str))

	local cmd, arg2 = addon:GetArgs(str, 2)
	cmd = strlower(cmd or "")
	arg2 = strlower(arg2 or "")

	if cmd == "" or cmd == "help" then
		addon:Printf("/mmgmm on       enable addon")
		addon:Printf("/mmgmm off      disable addon")
		addon:Printf("/mmgmm config   open addon options")
		addon:Printf("/mmgmm reset    reset options to default")
		addon:Printf("/mmgmm export   import/export options")
		addon:Printf("/mmgmm abc on   enable module abc")
		addon:Printf("/mmgmm abc off  disable module abc")
		addon:Printf("/mmgmm abc fire fire module abc (for tests)")
		addon:Printf("/mmgmm abc      toggle module abc")

		local modules = ""
		for modle in pairs(addon.modules) do
			if addon[modle].EnableMod then
				modules = modules .. " " .. modle:lower()
			end
		end
		addon:Printf("Modules: " .. modules)
		return
	end

	if cmd == "debug" then
		addon:ToggleDebug()
	elseif cmd == "on" then
		addon:EnableCmd()
	elseif cmd == "off" then
		addon:DisableCmd()
	elseif cmd == "reset" then
		for key, val in pairs(addon.Options.defaults.global) do
			addon.db.global[key] = val
		end
		addon.db.global.enabled = addon.db.global.enabled or false
		addon.isEnabled = addon.db.global.enabled
		addon:Printf(L["Addon options are reset to default values"])
	elseif cmd == "config" then
		InterfaceOptionsFrame_OpenToCategory(GetAddOnMetadata(addonName, "Title"))
		InterfaceOptionsFrame_OpenToCategory(GetAddOnMetadata(addonName, "Title"))
	elseif cmd == "export" then
		addon.GUI:Close()
		if addon.GUI.exportDisplay then
			addon.GUI.exportDisplay:Fire("OnClose")
		else
			addon.GUI:Export()
		end
	else
		for modle in pairs(addon.modules) do
			if cmd == modle:lower() and arg2 == "on" and addon.modules[modle].EnableMod then
					addon:DebugPrintf(" -> %s:EnableMod()", modle)
					addon.modules[modle]:EnableMod()
			elseif cmd == modle:lower() and arg2 == "off" and addon.modules[modle].DisableMod then
					addon:DebugPrintf(" -> %s:DisableMod()", modle)
					addon.modules[modle]:DisableMod()
			elseif cmd == modle:lower() and arg2 == "fire" and addon.modules[modle].ModuleIcon then
					addon:DebugPrintf(" -> Fire %s", modle)
					addon.GUI:Fire(modle)
			elseif cmd == modle:lower() then
				if addon.modules[modle].ToggleMod then
					addon:DebugPrintf(" -> %s:ToggleMod()", modle)
					addon.modules[modle]:ToggleMod()
				end
			end
		end
	end
end

-- called by AceAddon when Addon is fully loaded
function addon:OnInitialize()
	for modle in pairs(addon.modules) do
		addon[modle] = addon.modules[modle]
	end

	if DLAPI and DLAPI.SetFormat then DLAPI.SetFormat(addonName, "default") end
	addon:DebugPrintf("OnInitialize()")

	addon.handle = "mmgmm"
	addon.isDebug = false

	addon.timerSec = 1

	-- addon state flags
	addon.isEnabled = false
	addon.isInfight = false

	-- loads data and options
	addon.db = AceDB:New(addonName .. "DB", addon.Options.defaults, true)
	AceConfigRegistry:RegisterOptionsTable(addonName, addon.Options.GetOptions)
	local optionsFrame = AceConfigDialog:AddToBlizOptions(addonName, GetAddOnMetadata(addonName, "Title"))
	addon.optionsFrame = optionsFrame

	addon:RegisterChatCommand(addon.handle, "ChatCommand")
	addon:RegisterChatCommand(addon.handle .. "debug", function() addon:ChatCommand("debug"); end)
	addon:RegisterChatCommand(addon.handle .. "on", function() addon:ChatCommand("on"); end)
	addon:RegisterChatCommand(addon.handle .. "off", function() addon:ChatCommand("off"); end)
	addon:RegisterChatCommand(addon.handle .. "config", function() addon:ChatCommand("config"); end)
	addon:RegisterChatCommand(addon.handle .. "export", function() addon:ChatCommand("export"); end)

	addon.LDB = LDB:NewDataObject(addonName, {
		type = "data source",
		label = addonName,
		text = addon:GetLDBText(),
		icon = "Interface\\Icons\\Spell_Nature_Slow",
		OnEnter = function(this)
			local tooltip = LibQTip:Acquire(addon)
			tooltip:SmartAnchorTo(this)
			tooltip:SetAutoHideDelay(0.1, this)
			tooltip:EnableMouse(true)
			addon:Update()
			tooltip:Show()
		end,
		OnClick = function(this, button)
			if button == "RightButton" then
				InterfaceOptionsFrame_OpenToCategory(GetAddOnMetadata(addonName, "Title"))
			end
			if button == "LeftButton" then
				if addon.isEnabled then
					addon:Printf("OFF")
					addon.isEnabled = false
				else
					addon:Printf("ON")
					addon.isEnabled = true
				end
			end
			addon:Update()
		end,
		})
end

-- called by AceAddon on PLAYER_LOGIN
function addon:OnEnable()
	addon:DebugPrintf("OnEnable()")

	addon.db.global.enabled = addon.db.global.enabled or false
	addon.isEnabled = addon.db.global.enabled

	local disStr
	if addon.isEnabled then
		disStr = " |cff88ff88" .. L["enabled"] .. "|r, "
	else
		disStr = " |cffff8888" .. L["disabled"] .. "|r, "
	end

	addon:Printf("|cFF33FF99(" .. addon.METADATA.VERSION .. ")|r: " .. disStr ..
		format(L["enter /%s for help"], addon.handle, addon.handle))

	addon:DebugPrintf("Calling Login() in all modules")
	for modle in pairs(addon.modules) do
		if addon.modules[modle].Login then
			addon:DebugPrintf(" -> %s:Login()", modle)
			addon.modules[modle]:Login()
		end
	end

	addon.sectimer = C_Timer.NewTicker(addon.timerSec, function() addon:SecTimer() end)
	addon:Update()
end

function addon:GetLDBText(status, showAddonName)
	local showName = false or showAddonName

	if showName then
		return addonName .. (status and (": " .. tostring(status)) or "")
	else
		return status and (tostring(status)) or ""
	end
end

function addon:ReopenConfigDisplay(f)
	if InterfaceOptionsFrame:IsShown() then
		InterfaceOptionsFrame:Hide();
		InterfaceOptionsFrame_OpenToCategory(f)
	end
end

------------------------------------------------------------------------------
-- Timers

-- loop through all module timers once a second
function addon:SecTimer()
	for modle in pairs(addon.modules) do
		if addon.modules[modle].SecTimer then
			addon.modules[modle]:SecTimer()
		end
	end
end

function addon:Update()
	-- Header
	if addon.isEnabled then
		addon.LDB.text = addon:GetLDBText(L["ON"])
	else
		addon.LDB.text = addon:GetLDBText(L["OFF"])
	end

	for modle in pairs(addon.modules) do
		if addon.modules[modle].Update then
			addon:DebugPrintf(" -> %s:Update()", modle)
			addon.modules[modle]:Update()
		end
	end

	if not LibQTip:IsAcquired(addon) then
		return
	end

	-- Tooltip
	local tooltip = LibQTip:Acquire(addon)
	tooltip:Clear()
	tooltip:SetColumnLayout(2,"LEFT", "RIGHT")

	local line = tooltip:AddHeader()
	tooltip:SetCell(line, 1, "|cfffed100" .. addon.METADATA.NAME .. " (" .. addon.METADATA.VERSION .. ")", nil, "CENTER", 2)
	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, "")
	tooltip:SetCell(line, 2, "")
	tooltip:AddSeparator()
	for modle in pairs(addon.modules) do
		if addon[modle].ModuleName then
			line = tooltip:AddLine()
			tooltip:SetCell(line, 1, addon[modle]:ModuleName())
			tooltip:SetCell(line, 2, addon.db.global[modle] and L["enabled"] or L["disabled"])
		end
	end

	tooltip:AddSeparator()
	tooltip:AddLine(format(L["%sLeft-Click%s enables or disables"], ITEM_QUALITY_COLORS[5].hex, FONT_COLOR_CODE_CLOSE))
	tooltip:AddLine(format(L["%sRight-Click%s opens configuration"], ITEM_QUALITY_COLORS[5].hex, FONT_COLOR_CODE_CLOSE))
end

-- EOF
